;**********************************
; "Simple SEGA Master System BIOS" 
;  By David Senabre Albujer.
;
; Shows how to access cartridge from BIOS
; and how to run an inserted cartridge at user's will.
;
; Assembler: WLA_DX
;**********************************

;-------------------------------------------
; WLA-DX setup
;-------------------------------------------
.memorymap
defaultslot 0
slotsize $8000
slot 0 $0000
.endme

.rombankmap
bankstotal 1
banksize $8000
banks 1
.endro

;=====================================================
; variables
;=====================================================
.define ScrollY $C120
.define ScrollCont $C100
.define Mode $C130
.define DomesticMode $C131
.define NTSCMode $C132
.define CartIn $C133
.define GameIn $C134

; variables de estado del pad
.define Button1 $C140
.define Button2 $C141
.define Up $C142
.define Down $C143
.define Left $C144
.define Right $C145

;variables del sprite
.define BallYl $C200
.define BallYh $C201
.define BallXl $C202
.define BallXh $C203
.define BallTile $C204
.define BallSXl $C205
.define BallSXh $C206
.define BallSYl $C207
.define BallSYh $C208

; buffers (para leer o escribir segmentos de datos)
.define String1 $C210 ;16 bytes
.define String2 $C300 ;256bytes $C400

;--------------------
; SDSC  / cabecera
;--------------------
.sdsctag 1,"BIOS","SMS BIOS","David Senabre"

.include "SMSLib.inc"      ; Misc routines
.include "RamCode.inc"     ; RAM routines for cartridge access
.include "Prints.inc"      ; uses DeleteLine routine

.bank 0 slot 0
.org $0000
;-------------------------------------------
; Entry point
;-------------------------------------------
.section "Boot section" force
    di              ; desactivar interrupciones
    im 1            ; modo de interrupcion 1
    jp main         ; saltar al inicio del programa
.ends

.org $0038
;-------------------------------------------
; Rutina de interrupcion
;-------------------------------------------
.section "INT handler" force
    retn
.ends

.org $0066
;-------------------------------------------
; rutina de interrupcion del boton de pausa
;-------------------------------------------
.section "Pause handler" force
    retn
.ends

;=====================================================
; PROGRAMA
;=====================================================
.section "Main" free
main:
    ld sp, $dff0    ;poner la pila al final

    ;Borrar RAM $C000-DFFF
    ;--------------------
     ld bc,$C000    ;apuntar al comienzo de RAM
-:
     ld a,$0        ;vamos a escribir ceros
     ld (bc),a      ;escribir en la RAM
     inc c          ;apuntar a la siguiente direccion
     jp nz,-        ;repetir 256 veces
     inc b          ;y luego incremetar el byte alto
     ld a,$E0       ;hasta llegar a $E0
     cp b           ;eso significa $E000
     jp nz,-
     ; en $E000 ya no tenemos que escribir, porque la RAM
     ; acaba e $DFFF

    ;-------------------------------------------
    ; Inicializar registros del VDP y borrar VRAM
    ;-------------------------------------------
    call VDPInit
    call VRAMInit

    ;-------------------------------------------
    ; Cargar paleta
    ;-------------------------------------------
    ld hl,PaletteData               ; origen
    ld b,PaletteDataEnd-PaletteData ; tamao
    ld c,$00                        ; cargar desde posicion 0
    call LoadPalette

    ;-------------------------------------------
    ; Cargar tiles
    ;-------------------------------------------
    ; cargar las fuentes
    ld hl,0                         ; cargar desde posicion 0
    ld ix,FontData                  ; origen
    ld bc,160                       ; numero de tiles
    ld d,4                          ; bits por pixel
    call LoadTiles

    ;-------------------------------------------
    ; Escribir texto de introduccion en la pantalla
    ;-------------------------------------------
    ;empezamos a esceribir en la 3 linea
    ; cargamos la direccion de VRAM a escribir en hl
    ld hl,NameTableAddress+64*3
    call VRAMToHL   ;acceder a la direccion de VRAM indicada en hl
    ; ahora cargar la direccion del texto en hl, que usaremos
    ; como puntero para copiarlo a VRAM
    ld hl,Message
    -:
        ld a,(hl)    ; cargar nuevo byte a escribir desde (hl)
        cp $00       ; si es cero es que hemos llegado al final
        jp z,+       ;
;        sub $20      ; restamos 20 para que corresponda con ASCII
        out ($be),a  ; escribimos a VRAM
        ld a,$00
        out ($be),a  ; el 2 byte de cada tile lo dejamos a 0
        inc hl       ; apuntar al siguiente byte
        jp -
    +:

    ;-------------------------------------------
    ; activar la imagen
    ld a,%11000100
    out ($bf),a
    ld a,$81
    out ($bf),a
    ;-------------------------------------------
    ;detectar tipo de television
    call DetectTVType
    ld (NTSCMode),a
    call DetectMode
    ;-------------------------------------------
    ;esperar un tiempo antes de empezar a ejecutar
  ld a,0
  ld hl,$0
  -:
    call WaitForVBlankNoInt
    inc hl
    cp l
    jp nz, -
    ld a,1
  --:
    inc hl
    cp h
    jp nz, --

    ;------------------------------
    ; inicializar el bucle principal
    ;------------------------------
  ld bc,NameTableAddress   ;cargar inicio de NameTable
  ld hl,NameTableAddress   ;cargar inicio de NameTable
  call VRAMToHL            ;en la VRAM address
  ld a,0                   ;inicializar variables
  ld (ScrollY),a
  ld (Mode),a
  ld (CartIn),a
  ld (GameIn),a
  ld (BallSXl),a
  ld (BallSXh),a
  ld (BallSYl),a
  ld (BallSYh),a
  ld (BallYh),a
  ld (BallYl),a
  ld a,1
  ld (BallXh),a
  ld (BallXl),a
  ld (BallTile),a
  ld (BallSYl),a

    ;===============================
    ; Buble principal
    ;===============================
  Loop:
    call WaitForVBlankNoInt         ;esperar VBlank
    ld a,(Mode)                     ;ver el modo en que estamos
    cp 1
    jp z,+
    jp DoScroll                     ;modo=0, intro con scroll
    +:                              ;modo=1, menu de la BIOS
    jp DoMenu

    ;-------------------------
    ; MODO 0, intro con scroll
    ;-------------------------
    DoScroll:
    ld a,(ScrollCont)      ;incrementar contador de scroll
    inc a
    ld (ScrollCont),a
    cp $6                  ;cada 8 refrescos avanzar scroll un pixel
    jp nz,NoScroll         ;si no, pasar sin avanzar scroll

    ; Avanzar scroll
    ;------------------------------
    ld a,0
    ld (ScrollCont),a
    ld a,(ScrollY)
    inc a
    ld (ScrollY),a

    out ($bf),a             ; Registro de ScrollY
    push af
    ld a,$89
    out ($bf),a
            ;si hemos avanzado un multiplo de 8 (una linea)
            ;eliminar la linea que acaba de salir por arriba
    pop af
    push af
    and 7
    call z, DeleteLine   ;depende del registro bc
    pop af

    cp 224          ; Hemos llegado a la linea de pixels 224?
    jp nz,NoScroll  ; sino, seguir
                    ;pero si hemos llegado, es que ya ha pasado
                    ;todo el scrollY por la pantalla asi que
                    ;saltar al menu de la BIOS
    jp GoMode1

    ; Ya hemos acabado con el tratamiento del scroll en la presentacion
    ;-----------------------------------------------
  NoScroll:
    ld     a,(Button1)    ;comprobar si damos al START
    bit    0,a
    jp z,  Common         ;si boton 1 no pulsado, saltar
  GoMode1:
    call   GetPad1
    ld     a,(Button1)    ;comprobar que hemos soltado el boton
    bit    0,a            ;antes de seguir
    jp nz, GoMode1
    call   CargarMenu
    jp     Common
    ;-------------------------
    ; MODO 1, menu
    ;-------------------------
  DoMenu:
    ;los registros seran modificados en readcart
    ld       a,(Button1)    ;comprobar si damos al START
    bit      0,a
    call nz, RunCart
    ld       a,(Button2)    ;comprobar si damos al boton2
    bit      0,a
    jp z,    +
    -:
    call     GetPad1
    ld       a,(Button2)    ;comprobar que hemos soltado el boton antes de seguir
    bit      0,a
    jp nz,   -
    call     ScreenOff
    call     CheckCart
    call     PrintCart
    call     ScreenOn
    +:
    jp       Common

    ; Actualizar estado del PAD, actualizar sprites
    ;-----------------------------------------------
  Common:
    call     GetPad1
    call     MoveSprites
    call     RefreshSprites
    jp       Loop

.ends
.include "Engine.inc"   ; code for this particular ROM
                        ; pad reading, and sprite managing
;-------------------------------------------
; Data
;-------------------------------------------
.section "Data" FREE

Message:
.db " SEGA MASTER SYSTEM             "
.db " Demo BIOS Ver 1.0              "
.db "--------------------------------"
.db "                                "
.db "   Created by                   "
.db "   David Senabre Albujer (2006) "
.db "   www.ConsolasParaSiempre.net  ",0

Titulo:
.db "        -=SMS BIOS 1.0=-        "
.db " -=-=-=-=-=-=-=-=-=-=-=-=-=-=-  "
.db "    1)Run cartridge             "
.db "    2)Read cartridge            ",0
SistemaString:
.db "  System Information            "
.db "--------------------------------",0
DomesticString:
.db " Region: Japanese",0
ExportString:
.db " Region: USA-EUR",0
PALString:
.db " Video: PAL",0
NTSCString:
.db " Video: NTSC",0
CartString:
.db " Cartridge                      "
.db " --------",0
CartNotInString:
.db " Cartridge inserted: NO",0
CartInString:
.db " Cartridge inserted:YES",0
GameNotInString:
.db " Original game     : NO",0
GameInString:
.db " Original game     :YES",0


PaletteData:
.db $00,$3f,$3f,$38,$10,$3f,$04,$38,$00,$00,$00,$00,$00,$00,$00,$00 ; Black, White
.db $00,$04,$20,$38,$04,$20,$38,$38,$00,$00,$00,$00,$00,$00,$00,$00 ; Black, White
PaletteDataEnd:

; Datos de inicializacion de los registros del VDP
VdpData:
.db $04,$80,$84,$81,$ff,$82,$ff,$85,$ff,$86,$FF,$87,$00,$88,$00,$89,$ff,$8a
VdpDataEnd:

; resumencillo de algunos registros del VDP
;registro 80 = 04
;registro 81 = 84
;registro 82 = FF, direccion base de la name table
;registro 83 = --
;registro 84 = --
;registro 85 = FF, direccion base de tabla de sprites
;registro 86 = FF, los tiles de sprites tienen base en $2000
;registro 87 = FF, color de "borde"
;registro 88 = 00, scroll x
;registro 89 = 00, scroll y
;registro 8a = FF, desactivar interrupciones Line

FontData:
.incbin "ascii.chr"
FontDataEnd:


.ends